@@ -26,6 +26,9 @@
 #define __EMPATHY_MESSAGE_H__
 
 #include <glib-object.h>
+#ifdef ENABLE_TPL
+#include <telepathy-logger/log-entry.h>
+#endif
 
 #include "empathy-contact.h"
 #include "empathy-time.h"
@@ -52,8 +55,10 @@ struct _EmpathyMessageClass {
 };
 
 GType                    empathy_message_get_type          (void) G_GNUC_CONST;
-EmpathyMessage *         empathy_message_new_from_entry    (const gchar *message);
 EmpathyMessage *         empathy_message_new               (const gchar              *body);
+#ifdef ENABLE_TPL
+EmpathyMessage *         empathy_message_from_tpl_log_entry (TplLogEntry             *logentry);
+#endif /* ENABLE_TPL */
 TpChannelTextMessageType empathy_message_get_tptype        (EmpathyMessage           *message);
 void                     empathy_message_set_tptype        (EmpathyMessage           *message,
 							    TpChannelTextMessageType  type);
@@ -85,6 +90,10 @@ void                     empathy_message_set_id (EmpathyMessage *message, guint
 
 gboolean                 empathy_message_equal (EmpathyMessage *message1, EmpathyMessage *message2);
 
+TpChannelTextMessageFlags empathy_message_get_flags        (EmpathyMessage           *message);
+void                      empathy_message_set_flags        (EmpathyMessage           *message,
+							    TpChannelTextMessageFlags flags);
+
 G_END_DECLS
 
 #endif /* __EMPATHY_MESSAGE_H__ */
