@@ -1151,7 +1151,7 @@ tp_chat_got_remote_contact_cb (EmpathyTpContactFactory *factory,
 
 	if (error) {
 		DEBUG ("Error: %s", error->message);
-		empathy_tp_chat_close (EMPATHY_TP_CHAT (chat));
+		empathy_tp_chat_leave (EMPATHY_TP_CHAT (chat));
 		return;
 	}
 
@@ -1172,7 +1172,7 @@ tp_chat_got_self_contact_cb (EmpathyTpContactFactory *factory,
 
 	if (error) {
 		DEBUG ("Error: %s", error->message);
-		empathy_tp_chat_close (EMPATHY_TP_CHAT (chat));
+		empathy_tp_chat_leave (EMPATHY_TP_CHAT (chat));
 		return;
 	}
 
@@ -1505,7 +1505,7 @@ empathy_tp_chat_new (TpChannel *channel)
 			     NULL);
 }
 
-void
+static void
 empathy_tp_chat_close (EmpathyTpChat *chat) {
 	EmpathyTpChatPriv *priv = GET_PRIV (chat);
 
@@ -1672,25 +1672,25 @@ out:
 
 void
 empathy_tp_chat_acknowledge_messages (EmpathyTpChat *chat,
-				      const GList *messages) {
+				      const GSList *messages) {
 	EmpathyTpChatPriv *priv = GET_PRIV (chat);
 	/* Copy messages as the messges list (probably is) our own */
-	GList *msgs = g_list_copy ((GList *) messages);
-	GList *l;
+	GSList *msgs = g_slist_copy ((GSList *) messages);
+	GSList *l;
 	guint length;
 	GArray *message_ids;
 
 	g_return_if_fail (EMPATHY_IS_TP_CHAT (chat));
 	g_return_if_fail (priv->ready);
 
-	length = g_list_length ((GList *) messages);
+	length = g_slist_length ((GSList *) messages);
 
 	if (length == 0)
 		return;
 
 	message_ids = g_array_sized_new (FALSE, FALSE, sizeof (guint), length);
 
-	for (l = msgs; l != NULL; l = g_list_next (l)) {
+	for (l = msgs; l != NULL; l = g_slist_next (l)) {
 		GList *m;
 
 		EmpathyMessage *message = EMPATHY_MESSAGE (l->data);
@@ -1710,7 +1710,14 @@ empathy_tp_chat_acknowledge_messages (EmpathyTpChat *chat,
 		acknowledge_messages (chat, message_ids);
 
 	g_array_free (message_ids, TRUE);
-	g_list_free (msgs);
+	g_slist_free (msgs);
+}
+
+void
+empathy_tp_chat_acknowledge_all_messages (EmpathyTpChat *chat)
+{
+  empathy_tp_chat_acknowledge_messages (chat,
+    (GSList *) empathy_tp_chat_get_pending_messages (chat));
 }
 
 gboolean
@@ -1797,3 +1804,47 @@ empathy_tp_chat_can_add_contact (EmpathyTpChat *self)
 		tp_proxy_has_interface_by_id (priv->channel,
 			TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP);;
 }
+
+static void
+leave_remove_members_cb (TpChannel *proxy,
+			 const GError *error,
+			 gpointer user_data,
+			 GObject *weak_object)
+{
+	EmpathyTpChat *self = user_data;
+
+	if (error == NULL)
+		return;
+
+	DEBUG ("RemoveMembers failed (%s); closing the channel", error->message);
+	empathy_tp_chat_close (self);
+}
+
+void
+empathy_tp_chat_leave (EmpathyTpChat *self)
+{
+	EmpathyTpChatPriv *priv = GET_PRIV (self);
+	TpHandle self_handle;
+	GArray *array;
+
+	if (!tp_proxy_has_interface_by_id (priv->channel,
+		TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP)) {
+		empathy_tp_chat_close (self);
+		return;
+	}
+
+	self_handle = tp_channel_group_get_self_handle (priv->channel);
+	if (self_handle == 0) {
+		/* we are not member of the channel */
+		empathy_tp_chat_close (self);
+		return;
+	}
+
+	array = g_array_sized_new (FALSE, FALSE, sizeof (TpHandle), 1);
+	g_array_insert_val (array, 0, self_handle);
+
+	tp_cli_channel_interface_group_call_remove_members (priv->channel, -1, array,
+		"", leave_remove_members_cb, self, NULL, G_OBJECT (self));
+
+	g_array_free (array, TRUE);
+}
