@@ -356,7 +356,10 @@ main_window_error_edit_clicked_cb (GtkButton *button,
 	TpAccount *account;
 
 	account = g_object_get_data (G_OBJECT (button), "account");
-	empathy_accounts_dialog_show (GTK_WINDOW (window->window), account);
+
+	empathy_accounts_dialog_show_application (
+			gtk_widget_get_screen (GTK_WIDGET (button)), NULL, NULL,
+			account, FALSE, FALSE);
 
 	main_window_remove_error (window, account);
 }
@@ -404,7 +407,7 @@ main_window_error_display (EmpathyMainWindow *window,
 	}
 
 	info_bar = gtk_info_bar_new ();
-	gtk_info_bar_set_message_type (GTK_INFO_BAR (info_bar), GTK_MESSAGE_WARNING);
+	gtk_info_bar_set_message_type (GTK_INFO_BAR (info_bar), GTK_MESSAGE_ERROR);
 
 	gtk_widget_set_no_show_all (info_bar, TRUE);
 	gtk_box_pack_start (GTK_BOX (window->errors_vbox), info_bar, FALSE, TRUE, 0);
@@ -1059,7 +1062,8 @@ static void
 main_window_edit_accounts_cb (GtkAction         *action,
 			      EmpathyMainWindow *window)
 {
-	empathy_accounts_dialog_show (GTK_WINDOW (window->window), NULL);
+	empathy_accounts_dialog_show_application (gdk_screen_get_default (),
+			NULL, NULL, NULL, FALSE, FALSE);
 }
 
 static void
@@ -1107,7 +1111,9 @@ main_window_throbber_button_press_event_cb (GtkWidget         *throbber_ebox,
 		return FALSE;
 	}
 
-	empathy_accounts_dialog_show (GTK_WINDOW (window->window), NULL);
+	empathy_accounts_dialog_show_application (
+			gtk_widget_get_screen (GTK_WIDGET (throbber_ebox)),
+			NULL, NULL, NULL, FALSE, FALSE);
 
 	return FALSE;
 }
